@@ -6,13 +6,13 @@
     Test the autodoc extension.  This tests mainly the Documenters; the auto
     directives are tested in a test source file translated by test_build.
 
-    :copyright: Copyright 2007-2015 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2016 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
 # "raises" imported for usage by autodoc
-from util import TestApp, Struct, raises
-from nose.tools import with_setup
+from util import TestApp, Struct, raises, SkipTest
+from nose.tools import with_setup, eq_
 
 from six import StringIO
 from docutils.statemachine import ViewList
@@ -297,6 +297,9 @@ def test_get_doc():
         """Class docstring"""
         def __init__(self):
             """Init docstring"""
+
+        def __new__(cls):
+            """New docstring"""
     directive.env.config.autoclass_content = 'class'
     assert getdocl('class', C) == ['Class docstring']
     directive.env.config.autoclass_content = 'init'
@@ -380,6 +383,36 @@ def test_get_doc():
         directive.env.config.autoclass_content = 'both'
         assert getdocl('class', G) == ['Class docstring']
 
+    # class has __new__ method with docstring
+    # class docstring: depends on config value which one is taken
+    class H:
+        """Class docstring"""
+        def __init__(self):
+            pass
+
+        def __new__(cls):
+            """New docstring"""
+    directive.env.config.autoclass_content = 'class'
+    assert getdocl('class', H) == ['Class docstring']
+    directive.env.config.autoclass_content = 'init'
+    assert getdocl('class', H) == ['New docstring']
+    directive.env.config.autoclass_content = 'both'
+    assert getdocl('class', H) == ['Class docstring', '', 'New docstring']
+
+    # class has __init__ method without docstring and
+    # __new__ method with docstring
+    # class docstring: depends on config value which one is taken
+    class I:
+        """Class docstring"""
+        def __new__(cls):
+            """New docstring"""
+    directive.env.config.autoclass_content = 'class'
+    assert getdocl('class', I) == ['Class docstring']
+    directive.env.config.autoclass_content = 'init'
+    assert getdocl('class', I) == ['New docstring']
+    directive.env.config.autoclass_content = 'both'
+    assert getdocl('class', I) == ['Class docstring', '', 'New docstring']
+
 
 @with_setup(setup_test)
 def test_docstring_processing():
@@ -791,6 +824,7 @@ def test_generate():
                            'module', 'test_autodoc')
 
 # --- generate fodder ------------
+import six, sys
 
 __all__ = ['Class']
 
@@ -833,6 +867,7 @@ class Base(object):
     def inheritedmeth(self):
         """Inherited function."""
 
+
 class Class(Base):
     """Class to document."""
 
@@ -968,3 +1003,46 @@ class InstAttCls(object):
 
         self.ia2 = 'e'
         """Docstring for instance attribute InstAttCls.ia2."""
+
+
+def test_type_hints():
+    from sphinx.ext.autodoc import formatargspec
+    from sphinx.util.inspect import getargspec
+
+    try:
+        from typing_test_data import f0, f1, f2, f3, f4, f5, f6, f7, f8
+    except (ImportError, SyntaxError):
+        raise SkipTest('Cannot import Python code with function annotations')
+
+    def verify_arg_spec(f, expected):
+        eq_(formatargspec(f, *getargspec(f)), expected)
+
+    # Class annotations
+    verify_arg_spec(f0, '(x: int, y: numbers.Integral) -> None')
+
+    # Generic types with concrete parameters
+    verify_arg_spec(f1, '(x: typing.List[int]) -> typing.List[int]')
+
+    # TypeVars and generic types with TypeVars
+    verify_arg_spec(f2, '(x: typing.List[T],'
+                        ' y: typing.List[T_co],'
+                        ' z: T) -> typing.List[T_contra]')
+
+    # Union types
+    verify_arg_spec(f3, '(x: typing.Union[str, numbers.Integral]) -> None')
+
+    # Quoted annotations
+    verify_arg_spec(f4, '(x: str, y: str) -> None')
+
+    # Keyword-only arguments
+    verify_arg_spec(f5, '(x: int, *, y: str, z: str) -> None')
+
+    # Space around '=' for defaults
+    verify_arg_spec(f6, '(x: int = None, y: dict = {}) -> None')
+
+    # Callable types
+    verify_arg_spec(f7, '(x: typing.Callable[[int, str], int]) -> None')
+
+    # Tuple types
+    verify_arg_spec(f8, '(x: typing.Tuple[int, str],'
+                        ' y: typing.Tuple[int, ...]) -> None')
